@@ -486,8 +486,19 @@ kgmmuFaultBufferAllocSharedMemory_GH100
         return status;
     }
 
+    memdescSetPageSize(pMemDesc, AT_GPU, RM_PAGE_SIZE_HUGE);
     memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_131, 
                     pMemDesc);
+
+    if (status == NV_ERR_NO_MEMORY)
+    {
+        // TODO: Bug 5299603
+        NV_PRINTF(LEVEL_ERROR, "Allocation failed with big page size, retrying with default page size\n");
+        memdescSetPageSize(pMemDesc, AT_GPU, RM_PAGE_SIZE);
+        memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_131,
+                        pMemDesc);
+    }
+
     if (status != NV_OK)
     {
         goto destroy_memdesc;
@@ -541,7 +552,7 @@ kgmmuFaultBufferFreeSharedMemory_GH100
     pMemDesc = pClientShadowFaultBuffer->pFaultBufferSharedMemDesc;
 
     memdescUnmap(pMemDesc,
-                 NV_TRUE, osGetCurrentProcess(),
+                 NV_TRUE,
                  pClientShadowFaultBuffer->pFaultBufferSharedMemoryAddress,
                  pClientShadowFaultBuffer->pFaultBufferSharedMemoryPriv);
 
@@ -919,39 +930,6 @@ kgmmuIsReplayableShadowFaultBufferFull_GH100
             ((shadowBufPutIndex + 1) % maxBufferEntries)) ? NV_TRUE : NV_FALSE;
 }
 
-/*!
- * @brief Get the engine ID associated with the min CE
- *
- * @param[in] pKenrelGmmu  KernelGmmu object
- *
- * return engine ID of the min CE
- */
-NvU32
-kgmmuGetMinCeEngineId_GH100
-(
-    KernelGmmu *pKernelGmmu
-)
-{
-    return NV_PFAULT_MMU_ENG_ID_CE0;
-}
-
-/*!
- * @brief Get the engine ID associated with the max CE
- *
- * @param[in] pGpu         OBJGPU object
- * @param[in] pKenrelGmmu  KernelGmmu object
- *
- * return engine ID of the max CE
- */
-NvU32
-kgmmuGetMaxCeEngineId_GH100
-(
-    OBJGPU     *pGpu,
-    KernelGmmu *pKernelGmmu
-)
-{
-    return NV_PFAULT_MMU_ENG_ID_CE9;
-}
 
 /**
   * @brief  Sign extend a fault address to a supported width as per UVM requirements
@@ -1061,7 +1039,7 @@ kgmmuFillFakeSparseTables
         }
         else
         {
-            NvU64 nextLevelPD = pKernelGmmu->fakeSparseEntry[i-1];
+            NvU64 nextLevelPD = pKernelGmmu->fakeSparseEntry[i-1] & (~(NVBIT64(pFldAddr->shift) - 1ULL));
             portMemCopy(fillEntry.v8, NV_MMU_VER3_PDE__SIZE, templateFakeEntry.v8, NV_MMU_VER3_PDE__SIZE);
 
             // Update PDE address field to point to next level fake PD
